use JProCo;
GO
-- multiple anchor queries

--Cycle department with Cycle sub-departments.
;WITH DepartmentsCTE AS
( SELECT id, Department, parent, 0 as Level 
    FROM Departments
   WHERE id = 2
   UNION ALL
  SELECT d.id, d.Department, d.Parent,
         DepartmentsCTE.Level + 1 as Level 
    FROM Departments d
   INNER JOIN DepartmentsCTE
      ON DepartmentsCTE.id = d.Parent)
SELECT * 
  FROM DepartmentsCTE
 ORDER BY parent;

 
--Next, copy the anchor query and change the WHERE clause to filter on Department 4, 
--  which is Fitness.
SELECT id, Department, parent, 0 as Level 
  FROM Departments
 WHERE id = 4


--Then, drop it into the CTE query with another UNION ALL. Notice that there are now 
--   two UNION ALL statements, one between the two anchors and another between the 
--   anchors and the recursive query.
;WITH DepartmentsCTE AS
( SELECT id, Department, parent, 0 as Level 
    FROM Departments
   WHERE id = 2
   UNION ALL
SELECT id, Department, parent, 0 as Level 
  FROM Departments
 WHERE id = 4
   UNION ALL
  SELECT d.id, d.Department, d.Parent,
         DepartmentsCTE.Level + 1 as Level 
    FROM Departments d
   INNER JOIN DepartmentsCTE
      ON DepartmentsCTE.id = d.Parent)
SELECT * 
  FROM DepartmentsCTE
 ORDER BY parent;
 
--Figure 4.16 Cycle and Fitness departments including sub-departments.
--This multiple anchor CTE could have been created with a single anchor using 
--    the IN keyword in the WHERE clause like this:
WHERE id in (2, 4)

--This simpler query would produce the exact same results as the multiple anchors 
--   example. With recursive CTEs it is common to find multiple ways to achieve 
--   the same results.
